#include "global.h"
#include "lcd.h"
#include "timer.h"


unsigned char LCD_CMD;          // Command-Byte
unsigned char LCD_STATUS;       // Status-Byte
unsigned char LCD_DATA;         // Daten-Byte


/**** LCD-Display-Routinen **********************************************/

void Initlcd(void)
{
	unsigned int t;

    pd1  = 0xff;            // Port 4 als Ausgang
    pd0  = 0x00;            // Port 2 als Eingang
    pur0 = 0xff;            // Pull-Up's an

    p1   = 0xff;            // Port Pin's p1 auf High
    LCD_RESET = 0;          // Display Reset
	for (t=0; t<2000; t++)
	{}              // ca. 125s. Reset
    LCD_RESET = 1;          // Display Reset fertig

    LCD_DATA = 0x00;        // Text Home Adress setzen
    lcd_send_data();        // Auf 0000H setzen
    LCD_DATA = 0x00;
    lcd_send_data();
    LCD_CMD = 0x40;
    lcd_send_cmd();

    LCD_DATA = 0x14;        // Text AREA Adress setzen
    lcd_send_data();        // 14H = 20 Zeichen
    LCD_DATA = 0x00;
    lcd_send_data();
    LCD_CMD = 0x41;
    lcd_send_cmd();

    LCD_DATA = 0x00;        // Grafik Home Adress setzen
    lcd_send_data();        // Auf 0200H setzen
    LCD_DATA = 0x02;
    lcd_send_data();
    LCD_CMD = 0x42;
    lcd_send_cmd();

    LCD_DATA = 0x14;        // Grafik AREA Adress setzen
    lcd_send_data();        // 14H = 20 Zeichen
    LCD_DATA = 0x00;
    lcd_send_data();
    LCD_CMD = 0x43;
    lcd_send_cmd();

    LCD_DATA = 0x02;        // Offset Pointer setzen
    lcd_send_data();
    LCD_DATA = 0x00;
    lcd_send_data();
    LCD_CMD = 0x22;
    lcd_send_cmd();

    lcd_send_cmd();
    LCD_CMD = 0x80;         // OR-Mode
    lcd_send_cmd();

    LCD_CMD = 0x9C;         // Text an Grafik an
    lcd_send_cmd();

    LCD_DATA = 0x00;        // Adress Pointer setzen
    lcd_send_data();        // 0000H
    LCD_DATA = 0x00;
    lcd_send_data();
    LCD_CMD = 0x24;
    lcd_send_cmd();
    lcd_clear_ram();        // zum Schlu noch das Ram des
}                       // Display's lschen

void lcd_status(void)
{
	pd0 = 0x00;             // Port 2 als Eingang
    LCD_CD = 1;             // Commando High
    LCD_RD = 0;             // Read Low
    LCD_CE = 0;             // Chip Enable Low
    asm ("NOP");        
    asm ("NOP");        
    asm ("NOP");        
    LCD_STATUS = LCD_PORT;  // Status lesen
    LCD_CE = 1;             // Chip Enable High
    LCD_RD = 1;             // Read High
    LCD_STATUS &= (0x03);   // Bits ausmaskieren
}

void lcd_status_am(void)
{
	pd0 = 0x00;             // Port 2 als Eingang
    LCD_CD = 1;             // Commando High
    LCD_RD = 0;             // Read Low
    LCD_CE = 0;             // Chip Enable Low
    asm ("NOP");        
    asm ("NOP");        
    asm ("NOP");        
    LCD_STATUS = LCD_PORT;  // Status lesen
    LCD_CE = 1;             // Chip Enable High
    LCD_RD = 1;             // Read High
    LCD_STATUS &= (0x08);   // Bits ausmaskieren
}

void lcd_send_cmd(void)         // Commando an Display senden        
{
	do
	{
		lcd_status();
	}
	while (LCD_STATUS != 0x03);
    pd0 = 0xff;             // Port 2 als Ausgang
    asm ("NOP");        
    LCD_PORT = LCD_CMD;
    LCD_CD = 1;             // Commando High
    LCD_WR = 0;             // Write Low
    LCD_CE = 0;             // Chip Enable Low
    asm ("NOP");        
    asm ("NOP");        
    LCD_CE = 1;             // Chip Enable High
    LCD_WR = 1;             // Write High
}

void lcd_send_data(void)         // Daten an Display senden        
{
	do
	{
		lcd_status();
	}
	while (LCD_STATUS != 0x03);
    pd0 = 0xff;             // Port 2 als Ausgang
    LCD_PORT = LCD_DATA;
    LCD_CD = 0;             // Daten Low
    LCD_WR = 0;             // Write Low
    LCD_CE = 0;             // Chip Enable Low
    asm ("NOP");        
    asm ("NOP");        
    LCD_CE = 1;             // Chip Enable High
    LCD_WR = 1;             // Write High
}

void lcd_send_data_am(void)         // Daten an Display senden
{
	do
	{
		lcd_status_am();
	}
	while (LCD_STATUS != 0x08);
    pd0 = 0xff;             // Port 2 als Ausgang
    LCD_PORT = LCD_DATA;
    LCD_CD = 0;             // Daten Low
    LCD_WR = 0;             // Write Low
    LCD_CE = 0;             // Chip Enable Low
    asm ("NOP");        
    asm ("NOP");        
    LCD_CE = 1;             // Chip Enable High
    LCD_WR = 1;             // Write High
}

void lcd_pos(char x,char y)
{
	int i;
	i=y*20+x;
	LCD_DATA = LOBYTE(i);    // bergebene Adresse setzen
    lcd_send_data();        
    LCD_DATA = HIBYTE(i);
    lcd_send_data();
    LCD_CMD = 0x24;
    lcd_send_cmd();
}

void lcd_char(char c)
{
	LCD_DATA = c;
    lcd_send_data();     
    LCD_CMD = 0xC0;
    lcd_send_cmd();
}

void lcd_print( _far char * msg_string, char adress_x, char adress_y)
{
	int i;
	i=adress_y*20+adress_x;
	LCD_DATA = LOBYTE(i);    // bergebene Adresse setzen
    lcd_send_data();        
    LCD_DATA = HIBYTE(i);
    lcd_send_data();
    LCD_CMD = 0x24;
    lcd_send_cmd();
    LCD_CMD = 0xB0;         // Auto Write Mode an (Auto Inkrement)
    lcd_send_cmd();

    for (i=0; msg_string[i]; i++)
	{
		lcd_send_str(msg_string[i]);
	}
    LCD_CMD = 0xB2;         // Auto Write Mode aus (Auto Inkrement)
    lcd_send_cmd();
}

void lcd_send_str(unsigned char lcd_dat)
{
	lcd_dat -= 0x20;        // 20h subtrahieren da Display nich ASCII
    LCD_DATA = lcd_dat;     // kompatibel
    lcd_send_data_am();     // Daten an Display senden
}

void lcd_clear_ram(void)
{
	unsigned int t;
    LCD_DATA = 0x00;        // int. Ram des Display's lschen.
    lcd_send_data();        // laut Datenblatt des Herstellers 8kb
    LCD_DATA = 0x00;
    lcd_send_data();
    LCD_CMD = 0x24;
    lcd_send_cmd();
    LCD_CMD = 0xB0;         // Auto Write Mode an (Auto Inkrement)
    lcd_send_cmd();
	for (t=0; t<8192; t++)
	{
		LCD_DATA = 0x00;
        lcd_send_data_am();     // Daten an Display senden
	}
    LCD_CMD = 0xB2;         // Auto Write Mode aus (Auto Inkrement)
    lcd_send_cmd();
}

void lcd_pixel(char x,char y,char stat)
{
	int addr;
	char s;
	addr=0x200+(y*20+(x/8));
	LCD_DATA = LOBYTE(addr);    // bergebene Adresse setzen
    lcd_send_data();        
	LCD_DATA = HIBYTE(addr);
	lcd_send_data();
	LCD_CMD = 0x24;
    lcd_send_cmd();

	if(stat==0)
		s=0xF0;
	else
		s=0xF8;

	s+=(7-(x%8));
	LCD_CMD = s;
    lcd_send_cmd();
}

void lcd_lineX(unsigned char x,unsigned char y,unsigned char lx)
{
	unsigned char i;
	for(i=0;i<lx;i++)
		lcd_pixel(i+x,y,1);
}

void lcd_lineY(unsigned char x,unsigned char y,unsigned char ly)
{
	unsigned char i;
	for(i=0;i<ly;i++)
		lcd_pixel(x,i+y,1);
}

void lcd_line(signed char von_x,signed char von_y,signed char bis_x,signed char bis_y)
{
	signed char i,x_dist,y_dist,x,y;
	signed char dy;
	lcd_pixel(von_x,von_y,1);

	if(von_x > bis_x)  // tricky: I change the coordinates if the line goes from right to left
	{ 
		x = von_x; von_x = bis_x; bis_x = x;  
		y = von_y; von_y = bis_y; bis_y = y;
	}
	if(abs(bis_x - von_x) > abs(von_y - bis_y))
	{
		if(von_y < bis_y)
		{
			x_dist = bis_x - von_x;
			y_dist = bis_y - von_y;

			dy = (x_dist * 100) / y_dist;

			for(i = 0; i < x_dist; i++)
			{
				y = von_y + (i * 100) / dy;
				x = von_x + i;
				lcd_pixel(x,y,1);
			}
		}
		else
		{
			x_dist = bis_x - von_x;
			y_dist = von_y - bis_y;

			dy = (x_dist * 100) / y_dist;

			for(i = 0; i < x_dist; i++)
			{
				y = von_y - (i * 100) / dy;
				x = von_x + i;
				lcd_pixel(x,y,1);
			}
		}
	}
	else
	{
		if(von_y < bis_y)
		{
			x_dist = bis_x - von_x;
			y_dist = bis_y - von_y;

			dy = (y_dist * 100) / x_dist;

			for(i = 0; i < y_dist; i++)
			{
				x = von_x + (i * 100) / dy;
				y = von_y + i;
				lcd_pixel(x,y,1);
			}
		}
		else
		{
			x_dist = bis_x - von_x;
			y_dist = von_y - bis_y;

			dy = (y_dist * 100) / x_dist;

			for(i = 0; i < y_dist; i++)
			{
				x = von_x + (i * 100) / dy;
				y = von_y - i;
				lcd_pixel(x,y,1);
			}
		}
	}
}

void lcd_rectf(unsigned char x,unsigned char y,unsigned char lx,unsigned char ly)
{
	unsigned char i;
	for(i=0;i<ly;i++)
		lcd_lineX(x,i+y,lx);
}

void lcd_rect(unsigned char x,unsigned char y,unsigned char lx,unsigned char ly)
{
	lcd_lineX(x,y,lx);
	lcd_lineY(x,y,ly);
	lcd_lineX(x,y+ly,lx);
	lcd_lineY(x+lx,y,ly+1);
}